// VARS AND OBJECTS

// Generic serversent vars
global.posSave_buffer = buffer_create();
global.posSave_packetId = argument1;

// Networking consts
global.posSave_SAVE_POS = 200;
global.posSave_LOAD_POS = 199;
global.posSave_SUCCESS = 1;
global.posSave_FAIL = 0;

// Persistant vars
ini_open("gg2.ini");
global.posSave_saveKey = ini_read_real("Plugins", "posSave_saveKey", -1);
global.posSave_loadKey = ini_read_real("Plugins", "posSave_loadKey", -1);
ini_close();

// Objects

// Controller
global.posSave_Controller = object_add();
object_set_persistent(global.posSave_Controller, true);

// Particles
global.posSave_Particle = object_add();
global.posSave_ParticleSuccess = object_add();
object_set_parent(global.posSave_ParticleSuccess, global.posSave_Particle);
global.posSave_ParticleFail = object_add();
object_set_parent(global.posSave_ParticleFail, global.posSave_Particle);

// Menus
global.posSave_Menu = object_add();
object_set_parent(global.posSave_Menu, InGameMenuController);
object_set_depth(global.posSave_Menu, -130000);





// PLAYER ADDED ATTRIBUTES
// Player: enable plugin attributes
//
object_event_add(Player,ev_create,0,'
	posSave_x = -1;	// the saved position s x coordinate
	posSave_y = -1;	// the saved position s y coordinate
');

// done in ev_step too because according to lorgan server-sent plugins do weird things on startup
object_event_add(Player,ev_step,ev_step_begin,'
    if !variable_local_exists("posSave_x") {
		posSave_x = -1;
		posSave_y = -1;
    }
');


// PLAYER CONTROL ADDITIONS
// generate controller
//
object_event_add(PlayerControl,ev_step,ev_step_begin,'
	object_exists(global.posSave_Controller) {
		if(not instance_exists(global.posSave_Controller)) {
			instance_create(0,0,global.posSave_Controller);
		}
	}
');




// GENERIC MENU SCRIPT
//
//
if !variable_global_exists("pluginOptions") {
    global.pluginOptions = object_add();
    object_set_parent(global.pluginOptions,OptionsController);  
    object_set_depth(global.pluginOptions,-130000);
    object_event_add(global.pluginOptions,ev_create,0,'   
        menu_create(40, 140, 300, 200, 30);
		
        if (room != Options)
            menu_setdimmed();
			
        menu_addback("Back", "
            instance_destroy();
            if(room == Options)
                instance_create(0,0,MainMenuController);
            else
                instance_create(0,0,InGameMenuController);
        ");
    ');
    
    object_event_add(InGameMenuController,ev_create,0,'
        menu_addlink("Plugin Options", "
            instance_destroy();
            instance_create(0,0,global.pluginOptions);
        ");
    ');
} 

object_event_add(global.pluginOptions, ev_create, 0, '
	menu_addlink("Position save", "
		instance_destroy();
		instance_create(0,0,global.posSave_Menu);
	");
');


// SAVEPOS MENU
//
//
object_event_add(global.posSave_Menu,ev_create,0,'   
	menu_create(40, 140, 300, 200, 30);
	menu_setdimmed();
	
	menu_addlink("Save current position", global.posSave_saveScript);
	menu_addlink("Load saved position", global.posSave_loadScript);
	menu_addlink("","");
	
    menu_addedit_key("Save pos shortcut", "global.posSave_saveKey");
    menu_addedit_key("Load pos shortcut", "global.posSave_loadKey");
		
	menu_addback("Back", "
		instance_destroy();
		instance_create(0,0,global.pluginOptions);
	");
');

// save persistent data on menu exit
object_event_add(global.posSave_Menu,ev_destroy,0,'
	ini_open("gg2.ini");
    ini_write_real("Plugins", "posSave_saveKey", global.posSave_saveKey);
    ini_write_real("Plugins", "posSave_loadKey", global.posSave_loadKey);
	ini_close();
');



// NETWORKING SCRIPTS
// what to do when this or that byte is received, as client or host
//
object_event_add(global.posSave_Controller, ev_step, ev_step_normal, '
	var receiveBuffer;
	receiveBuffer = PluginPacketGetBuffer(global.posSave_packetId);
	
	while(receiveBuffer != -1) {
		var data;
		data = read_ubyte(receiveBuffer);
	
		paramPlayer = PluginPacketGetPlayer(global.posSave_packetId);
		switch(data) {
			case global.posSave_SAVE_POS:
				if(global.isHost) {
					event_user(2);
				}
				break;
			case global.posSave_LOAD_POS:
				if(global.isHost) {
					event_user(3);
				}
				break;
			case global.posSave_SUCCESS:
				var paramX;
				paramX = read_ushort(receiveBuffer);
				var paramY;
				paramY = read_ushort(receiveBuffer);
				instance_create(paramX, paramY, global.posSave_ParticleSuccess);
				global.myself.posSave_x = paramX;
				global.myself.posSave_y = paramY;
				break;
			case global.posSave_FAIL:
				instance_create(read_ushort(receiveBuffer), read_ushort(receiveBuffer), global.posSave_ParticleFail);
				break;
		}
		
		paramPlayer = noone;
		// get next data
		buffer_clear(receiveBuffer);
		PluginPacketPop(global.posSave_packetId);
		receiveBuffer = PluginPacketGetBuffer(global.posSave_packetId);
	}
');


// NETWORKING SCRIPT BITS

// This script inits a Buffer <sendBuffer>.
global.posSave_netScriptBegin = '
	var sendBuffer;
	sendBuffer = buffer_create();
';

// This script sends the Buffer <sendBuffer> using PluginPacketSend and destroys it.
global.posSave_netScriptSend = '
	PluginPacketSend(global.posSave_packetId, sendBuffer);
	buffer_clear(sendBuffer);
	buffer_destroy(sendBuffer);
';

// This script sends the Buffer <sendBuffer> to <paramPlayer> using PluginPacketSendTo, and destroys the buffer.
global.posSave_netScriptSendTo = '
	PluginPacketSendTo(global.posSave_packetId, sendBuffer, paramPlayer);
	buffer_clear(sendBuffer);
	buffer_destroy(sendBuffer);
';



// CONTROLLER FUNCTIONS
//
 
// LOAD SCRIPT
//
// This function loads <paramPlayer> s saved position (calling posSave.Controller.eventUser(1))
// If host, <paramPlayer> is always global.myself (the host s player) and calls posSave.Controller.eventUser(3)
global.posSave_loadScript = '
	with(global.posSave_Controller) {
		if(global.isHost) {
			paramPlayer = global.myself;
			event_user(3);
			paramPlayer = noone;
		} else {
			event_user(1);
		}
	}
';

// SAVE SCRIPT
//
// This function saves <paramPlayer> s saved position (calling posSave.Controller.eventUser(0))
// If host, <paramPlayer> is always global.myself (the host s player) and calls posSave.Controller.eventUser(2)
global.posSave_saveScript = '
	with(global.posSave_Controller) {
		if(global.isHost) {
			paramPlayer = global.myself;
			event_user(2);
			paramPlayer = noone;
		} else {
			event_user(0);
		}
	}
';


// CONTROLLER ATTRIBUTES
//
//
object_event_add(global.posSave_Controller, ev_create, 0, '
	paramPlayer = noone;						// used in networking + event_user 2 & 3
	alarm[0] = 60 * global.game_fps / 30;		// alarm that shows your saved position
');


// CONTROLLER ALARM
// alarm 0 shows your saved position
//
object_event_add(global.posSave_Controller, ev_alarm, 0, '
	if(global.myself.posSave_x != -1) {
		instance_create(global.myself.posSave_x, global.myself.posSave_y, global.posSave_ParticleSuccess);
	}
	alarm[0] = 60 * global.game_fps / 30;
');




// KEYBOARD EVENTS
//
//
object_event_add(global.posSave_Controller, ev_keypress, 0, '
	if(not instance_exists(InGameMenuController)) {
		if(global.posSave_saveKey != -1) {
			if(keyboard_check_pressed(global.posSave_saveKey) || keyboard_check_released(global.posSave_saveKey)) {
				' + global.posSave_saveScript + '
			}
		}
		if(global.posSave_loadKey != -1) {
			if(keyboard_check_pressed(global.posSave_loadKey) || keyboard_check_released(global.posSave_loadKey)) {
				' + global.posSave_loadScript + '
			}
		}
	}
');




// USER EVENTS
//
// Say to server you re saving your pos
object_event_add(global.posSave_Controller, ev_other, ev_user0, 
	global.posSave_netScriptBegin + '
	
		write_ubyte(sendBuffer, global.posSave_SAVE_POS);
		
	' + global.posSave_netScriptSend
);


// Say to server you re loading your pos
object_event_add(global.posSave_Controller, ev_other, ev_user1, 
	global.posSave_netScriptBegin + '
	
		write_ubyte(sendBuffer, global.posSave_LOAD_POS);
		
	' + global.posSave_netScriptSend
);


// As server, save a pos
object_event_add(global.posSave_Controller, ev_other, ev_user2, '
	if(paramPlayer.object != -1) {
		// if player is immobile
		if(paramPlayer.object.speed <= 1 && paramPlayer.object.onground) {
			// save his pos
			paramPlayer.posSave_x = paramPlayer.object.x;
			paramPlayer.posSave_y = paramPlayer.object.y;
			
			// and send him approval + actual saved pos
			' + global.posSave_netScriptBegin + '
			write_ubyte(sendBuffer, global.posSave_SUCCESS);
			write_ushort(sendBuffer, paramPlayer.object.x);
			write_ushort(sendBuffer, paramPlayer.object.y);
			' + global.posSave_netScriptSendTo + '
			
		} else {
		
			// else, send fail indicator (plus fail position)
			' + global.posSave_netScriptBegin + '
			write_ubyte(sendBuffer, global.posSave_FAIL);
			write_ushort(sendBuffer, paramPlayer.object.x);
			write_ushort(sendBuffer, paramPlayer.object.y);
			' + global.posSave_netScriptSendTo + '
		}
	}
');


// As server, load a pos
object_event_add(global.posSave_Controller, ev_other, ev_user3, '
	if(paramPlayer.object != -1) {
		with(paramPlayer) {
			if(!object.intel) {
				object.speed = 0;
				object.x = posSave_x;
				object.y = posSave_y;
			}
		}
	}
');



// ROOM CHANGES
// Reset players pos to prevent being stuck in impossible places and shit
//
object_event_add(RoomChangeObserver, ev_other, ev_room_start, '
	with(Player) {
		posSave_x = -1;
		posSave_y = -1;
	}
');



// PARTICLES
//
//
object_event_add(global.posSave_Particle, ev_create, 0, '
	lifeTime = 15 * global.game_fps / 30;
	lifeTimeLeft = lifeTime;
	width = 20;
');

object_event_add(global.posSave_Particle, ev_step, ev_step_begin, '
	lifeTimeLeft -= 1;
	if(lifeTimeLeft == 0) {
		instance_destroy();
	}
');

object_event_add(global.posSave_ParticleSuccess, ev_draw, 0, '
	draw_set_alpha(lifeTimeLeft / lifeTime * 2);
	draw_set_color(c_blue);
	draw_circle(x, y, width * (lifeTimeLeft / lifeTime), true);
');

object_event_add(global.posSave_ParticleFail, ev_draw, 0, '
	draw_set_alpha(lifeTimeLeft / lifeTime * 2);
	draw_set_color(c_red);
	draw_set_halign(fa_center);
	draw_text(x, y, "SAVEPOS#FAILED!");
');